from .vars import *
from .main import *
from .jobs import *
from .forms import *
from .hooks import *
from .files import *
from .packages import *
from .public import *
from .dashes import *
from .subdomains import *
from .memberships import *



import os, json, requests, websocket as ws
from functools import lru_cache
from abstra_cli.credentials import get_auth_headers


@lru_cache(maxsize=None)
def get_info_from_token(api_token):
    headers = get_auth_headers(api_token)
    response = requests.get(f"https://auth.abstra.cloud/abstra-cloud", headers=headers)
    response_json = response.json()
    if response_json is None:
        return None, None

    author_id = response_json.get("author_id")
    workspaces = response_json.get("workspaces", [{}])

    workspace_id = None
    if len(workspaces) != 0:
        workspace_id = workspaces[0].get("id")

    return workspace_id, author_id


def usage(fname, args, kwargs, author_id, workspace_id):
    if os.getenv("DISABLE_USAGE_STATISTICS"):
        return

    try:
        requests.post(
            "https://usage-api.abstra.cloud/api/rest/cli-usage",
            data=json.dumps(
                {
                    "author_id": author_id,
                    "workspace_id": workspace_id,
                    "method": fname,
                    "arguments": {"args": args[1:], "kwargs": list(kwargs.keys())},
                }
            ),
            headers={"content-type": "application/json"},
        )
    except Exception as e:
        pass


def wait_for_api_token(cli_uuid):
    ws_url = f"wss://pubsub.abstra.cloud/external/sub/cli-login-{cli_uuid}"
    ws_client = None
    try:
        ws_client = ws.create_connection(ws_url)
        result = ws_client.recv()
        if result is None:
            return None
        return json.loads(result).get("api_token")
    except Exception as e:
        return None
    finally:
        ws_client and ws_client.close()



from abstra_cli.checkers import credentials_check
from abstra_cli.apis import get_file_signed_url


def get_file_url(filepath):
    credentials_check("get_file_url", filepath)
    return get_file_signed_url(filepath)



import fire

from abstra_cli.deploy import deploy
from abstra_cli.login import configure, login
import abstra_cli.messages as messages
import abstra_cli.checkers as checkers
from abstra_cli.resources import (
    Forms,
    Files,
    Packages,
    Vars,
    Hooks,
    Jobs,
    Dashes,
    Subdomains,
)


class CLI(object):
    """
    A CLI to manage your Abstra Cloud Workspace environment.

    usage: abstra <command> <resource> [<argument> ...] [parameters]
    """

    def configure(self, api_token=None):
        configure(api_token)
        checkers.configuration_check("configure", api_token)

    def login(self):
        login()
        checkers.configuration_check("login")

    def deploy(self, **kwargs):
        should_login = kwargs.get("login", False)
        if not checkers.is_logged() and should_login:
            login()

        checkers.credentials_check("deploy", **kwargs)
        deploy(**kwargs)

    def list(self, resource, **kwargs):
        checkers.credentials_check("list", resource, **kwargs)
        list_func = {
            "vars": Vars.list,
            "jobs": Jobs.list,
            "files": Files.list,
            "forms": Forms.list,
            "hooks": Hooks.list,
            "dashes": Dashes.list,
            "packages": Packages.list,
            "subdomain": Subdomains.list,
        }.get(resource, messages.not_implemented)

        list_func()

    def add(self, resource, *args, **kwargs):
        checkers.credentials_check("add", resource, *args, **kwargs)
        add_func = {
            "vars": Vars.add,
            "dash": Dashes.add,
            "dashes": Dashes.add,
            "form": Forms.add,
            "forms": Forms.add,
            "hook": Hooks.add,
            "hooks": Hooks.add,
            "job": Jobs.add,
            "jobs": Jobs.add,
            "files": Files.add,
            "packages": Packages.add,
        }.get(resource, messages.not_implemented)

        add_func(*args, **kwargs)

    def update(self, resource, *args, **kwargs):
        checkers.credentials_check("update", resource, *args, **kwargs)
        update_func = {
            "dash": Dashes.update,
            "dashes": Dashes.update,
            "form": Forms.update,
            "forms": Forms.update,
            "hook": Hooks.update,
            "hooks": Hooks.update,
            "job": Jobs.update,
            "jobs": Jobs.update,
            "subdomain": Subdomains.update,
        }.get(resource, messages.not_implemented)

        update_func(*args, **kwargs)

    def remove(self, resource, *args, **kwargs):
        checkers.credentials_check("remove", resource, *args, **kwargs)
        remove_func = {
            "vars": Vars.remove,
            "dash": Dashes.remove,
            "dashes": Dashes.remove,
            "form": Forms.remove,
            "forms": Forms.remove,
            "hook": Hooks.remove,
            "hooks": Hooks.remove,
            "job": Jobs.remove,
            "jobs": Jobs.remove,
            "files": Files.remove,
            "packages": Packages.remove,
        }.get(resource, messages.not_implemented)

        remove_func(*args, **kwargs)

    def play(self, resource, *args, **kwargs):
        checkers.credentials_check("play", resource, *args, **kwargs)
        play_func = {
            "dash": Dashes.play,
            "form": Forms.play,
            "hook": Hooks.play,
            "forms": Forms.play,
            "hooks": Hooks.play,
        }.get(resource, messages.not_implemented)

        play_func(*args, **kwargs)

    # Aliases
    def upload(self, *args, **kwargs):
        if not len(args):
            args = ["."]
        self.add("files", *args, **kwargs)

    def ls(self, *args, **kwargs):
        self.list("files", *args, **kwargs)

    def rm(self, *args, **kwargs):
        self.remove("files", *args, **kwargs)

    def install(self, *args, **kwargs):
        self.add("packages", *args, **kwargs)


def _SeparateFlagArgs(args):
    try:
        index = args.index("--help")
        args = args[:index]
        return args, ["--help"]
    except ValueError:
        return args, []


def main():
    fire.core.parser.SeparateFlagArgs = _SeparateFlagArgs
    fire.Fire(CLI)


if __name__ == "__main__":
    main()



import sys
from crontab import CronTab
from abstra_cli.resources.resources import Resource
import abstra_cli.messages as messages
import abstra_cli.apis as apis


NAME_PARAMETERS = ["name", "title"]
IDT_PARAMETERS = ["identifier", "idt"]
CODE_PARAMETERS = ["file", "f", "code", "c"]
SCHEDULE_PARAMETERS = ["schedule", "crontab"]
NON_FLAG_PARAMETERS = (
    NAME_PARAMETERS + IDT_PARAMETERS + CODE_PARAMETERS + SCHEDULE_PARAMETERS
)
FLAG_PARAMETERS = ["enabled"]
JOB_PARAMETERS = FLAG_PARAMETERS + NON_FLAG_PARAMETERS


def check_valid_parameters(parameters: dict) -> None:
    for param in parameters.keys():
        if param not in JOB_PARAMETERS:
            messages.invalid_parameter(param)
            sys.exit(1)
    for param, value in parameters.items():
        if param in NON_FLAG_PARAMETERS and value in [True, False]:
            messages.invalid_non_flag_parameter_value(param)
            sys.exit(1)


def evaluate_parameter_name(parameters: dict, use_default=True) -> dict:
    name = parameters.get("name") or parameters.get("n") or parameters.get("title")
    if not name and not use_default:
        return {}
    return {"name": name or "New Job"}


def evaluate_parameter_idt(parameters: dict) -> dict:
    identifier = parameters.get("idt") or parameters.get("identifier")
    if not identifier:
        return {}
    return {"identifier": identifier}


def evaluate_parameter_schedule(parameters: dict, use_default=True) -> dict:
    schedule = parameters.get("schedule") or parameters.get("crontab")
    if schedule:
        try:
            CronTab(schedule)
            return {"schedule": schedule}
        except:
            print("Bad crontab expression")
            sys.exit(1)

    return {"schedule": "00 00 00 * *"} if use_default else {}


def evaluate_parameters_file_and_code(parameters: dict, use_default=True) -> dict:
    EMPTY_JOB = "print('Done!')"
    file = parameters.get("file") or parameters.get("f")
    code = parameters.get("code") or parameters.get("c")

    if file and code:
        messages.code_and_file_not_allowed()
        sys.exit(1)

    if file:
        with open(file, "r") as f:
            return {"code": f.read()}

    if code:
        return {"code": code}

    return {"code": EMPTY_JOB} if use_default else {}


def evaluate_flag_parameters(parameters: dict) -> dict:
    evaluated_params = {}
    for param, value in parameters.items():
        if param in FLAG_PARAMETERS:
            if value == "true" or value == True:
                evaluated_params[param] = True
                continue
            if value == "false" or value == False:
                evaluated_params[param] = False
                continue

            messages.invalid_flag_parameter_value(param)
            sys.exit(1)
    return evaluated_params


class Jobs(Resource):
    @staticmethod
    def list():
        jobs = apis.list_workspace_jobs()
        messages.print_jobs(jobs)

    @staticmethod
    def add(*args, **kwargs):
        upsert = kwargs.pop("upsert", False)
        idt = evaluate_parameter_idt(kwargs).get("identifier")
        if upsert and not idt:
            messages.upsert_without_identifier("identifier")
            sys.exit(1)

        check_valid_parameters(kwargs)

        job_data = {
            **evaluate_parameter_name(kwargs),
            **evaluate_parameter_idt(kwargs),
            **evaluate_parameter_schedule(kwargs),
            **evaluate_parameters_file_and_code(kwargs),
            **evaluate_flag_parameters(kwargs),
        }

        if job_data:
            try:
                if upsert:
                    apis.upsert_workspace_job(job_data)
                    messages.upserted_message("Job", idt)
                else:
                    idt = apis.add_workspace_job(job_data)["identifier"]
                    messages.created_message("Job", idt)
            except Exception as e:
                print(e)
                messages.create_failed(
                    "Job",
                )
                sys.exit(1)

    @staticmethod
    def update(*args, **kwargs):
        if not len(args):
            messages.required_argument("identifier")
            sys.exit(1)
        idt = args[0]

        if not len(kwargs):
            messages.missing_parameters_to_update("job", idt)
            sys.exit(1)

        check_valid_parameters(kwargs)

        job_data = {
            **evaluate_parameter_name(kwargs, use_default=False),
            **evaluate_parameter_idt(kwargs),
            **evaluate_parameter_schedule(kwargs, use_default=False),
            **evaluate_parameters_file_and_code(kwargs, use_default=False),
            **evaluate_flag_parameters(kwargs),
        }

        if job_data:
            try:
                apis.update_workspace_job(idt, job_data)
                messages.updated_message("Job", idt)
            except Exception as e:
                print(e)
                messages.update_failed("Job", idt)
                sys.exit(1)

    @staticmethod
    def remove(*args, **kwargs):
        if not len(args):
            messages.required_argument("identifier")
            sys.exit(1)

        idt = args[0]
        apis.delete_workspace_job(idt)
        messages.deleted_message("Job", idt)
